﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RDNIDWRAPPER;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Diagnostics;
using System.Threading;

namespace NIDCardCS
//NID Card CS.net
//V1.0.0.2081 For TDK Series DLL V2.21.7D+, support DLD
//V1.0.0.2082 fix bug
//....
//....
//....
//V1.0.0.2091 Code clearing
//V1.0.0.2092 Correct program name
//V1.2.31.1   For Support RDNIDLib.dll v.2.31

{
    public partial class Form1 : Form
    {
        string StartupPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location);

        public static string GetCurrentExecutingDirectory(System.Reflection.Assembly assembly)
        {
            string filePath = new Uri(System.Reflection.Assembly.GetExecutingAssembly().CodeBase).LocalPath;
            return Path.GetDirectoryName(filePath);
        }

        public Form1()
        {
            InitializeComponent();

            string fileName = StartupPath + "\\RDNIDLib.DLX";
            if (System.IO.File.Exists(fileName) == false)
            {
                MessageBox.Show("RDNIDLib.DLX not found");
            }

            System.Version version = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version;


            this.Text = String.Format("R&D NID Card C# V{0}.{1}.{2}.{3}", version.Major, version.Minor, version.Build, version.Revision);

            IntPtr mNIDLibHandle = (IntPtr)0;
            byte[] _lic = String2Byte(fileName);


            int nres = 0;
          
            nres = RDNIDLib.openNIDLibRD(_lic);                     //for openLib 
            if (nres != 0)
            {
                String m;
                m = String.Format("Error: {0} ", nres);
                MessageBox.Show(m);
            }

            byte[] Licinfo = new byte[1024];

            RDNIDLib.getLicenseInfoRD(Licinfo);
            
            m_lblDLDInfo.Text = "License Info: " + aByteToString(Licinfo);

            byte[] Softinfo = new byte[1024];
            RDNIDLib.getSoftwareInfoRD(Softinfo);
            m_lblSoftwareInfo.Text = "RDNIDLib Info: " + aByteToString(Softinfo);

            byte[] ZDLibinfo = new byte[1024];          
   

            ListCardReader();

            lblStatus.Text = "Status: Ready";

        }

        public int selectReader(String reader)
        {
            byte[] _reader = String2Byte(reader);
            int res = RDNIDLib.selectReaderRD(_reader);
            return res;
        }

        private void ListCardReader()
        {
            byte[] szReaders = new byte[1024 * 2];
            int size = szReaders.Length;
            int numreader = RDNIDLib.getReaderListRD(szReaders, size);
            if (numreader <= 0)
                return;
            String s = aByteToString(szReaders);
            String[] readlist = s.Split(';');
            if (readlist != null)
            {
                for (int i = 0; i < readlist.Length; i++)
                    m_ListReaderCard.Items.Add(readlist[i]);
                m_ListReaderCard.SelectedIndex = 0;
            }
        }

        static string aByteToString(byte[] b)
        {
            Encoding ut = Encoding.GetEncoding(874); // 874 for Thai langauge
            int i;
            for (i = 0; b[i] != 0; i++) ;

            string s = ut.GetString(b);
            s = s.Substring(0, i);
            return s;
        }

        static byte[] String2Byte(string s)
        {
            // Create two different encodings.
            Encoding ascii = Encoding.GetEncoding(874);
            Encoding unicode = Encoding.Unicode;

            // Convert the string into a byte array.
            byte[] unicodeBytes = unicode.GetBytes(s);

            // Perform the conversion from one encoding to the other.
            byte[] asciiBytes = Encoding.Convert(unicode, ascii, unicodeBytes);

            return asciiBytes;
        }

        String _yyyymmdd_(String d)
        {
            string s = "";
            string _yyyy = d.Substring(0, 4);
            string _mm = d.Substring(4, 2);
            string _dd = d.Substring(6, 2);


            string[] mm = { "", "ม.ค.", "ก.พ.", "มี.ค.", "เม.ย.", "พ.ค.", "มิ.ย.", "ก.ค.", "ส.ค.", "ก.ย.", "ต.ค.", "พ.ย.", "ธ.ค." };
            string _tm = "-";
            if (_mm == "00")
            {
                _tm = "-";
            }
            else
            {
                _tm = mm[int.Parse(_mm)];
            }

            if (_yyyy == "0000")
                _yyyy = "-";

            if (_dd == "00")
                _dd = "-";

            s = _dd + " " + _tm + " " + _yyyy;
            return s;
        }

        enum NID_FIELD
        {
            NID_Number,   //1234567890123#

            TITLE_T,    //Thai title#
            NAME_T,     //Thai name#
            MIDNAME_T,  //Thai mid name#
            SURNAME_T,  //Thai surname#

            TITLE_E,    //Eng title#
            NAME_E,     //Eng name#
            MIDNAME_E,  //Eng mid name#
            SURNAME_E,  //Eng surname#

            HOME_NO,    //12/34#
            MOO,        //10#
            TROK,       //ตรอกxxx#
            SOI,        //ซอยxxx#
            ROAD,       //ถนนxxx#
            TUMBON,     //ตำบลxxx#
            AMPHOE,     //อำเภอxxx#
            PROVINCE,   //จังหวัดxxx#

            GENDER,     //1#			//1=male,2=female

            BIRTH_DATE, //25200131#	    //YYYYMMDD 
            ISSUE_PLACE,//xxxxxxx#      //
            ISSUE_DATE, //25580131#     //YYYYMMDD 
            EXPIRY_DATE,//25680130      //YYYYMMDD 
            ISSUE_NUM,  //12345678901234 //14-Char
            END
        };

        Stopwatch speedReadText;
        Stopwatch speedReadAll;

        protected int ReadCard()
        {
            speedReadAll = new Stopwatch();
            speedReadText = new Stopwatch();

            speedReadAll.Start();
            speedReadText.Start();

            String strTerminal = m_ListReaderCard.GetItemText(m_ListReaderCard.SelectedItem);
            int obj = selectReader(strTerminal);
            if (obj < 0)
            {
                String m;
                m = String.Format("Error: {0} ", obj);

                lblStatus.Text = "Status: Error " + obj;
                MessageBox.Show(m, Application.ProductName);

                RDNIDLib.disconnectCardRD(obj);
                RDNIDLib.deselectReaderRD(obj);
                return obj;
            }

            Int32 nInsertCard = 0;
            nInsertCard = RDNIDLib.connectCardRD(obj);
            if (nInsertCard != DefineConstants.NID_SUCCESS)
            {
                String m;
                m = String.Format("Error: {0} ", nInsertCard);

                lblStatus.Text = "Status: Error " + nInsertCard;
                MessageBox.Show(m, Application.ProductName);

                RDNIDLib.disconnectCardRD(obj);
                RDNIDLib.deselectReaderRD(obj);
                return nInsertCard;
            }


            byte[] data = new byte[1024];
            int res = RDNIDLib.getNIDTextRD(obj, data, data.Length);
            if (res != DefineConstants.NID_SUCCESS)
            {
                String m;
                m = String.Format("Error: {0} ", res);

                lblStatus.Text = "Status: Error " + res;
                MessageBox.Show(m, Application.ProductName);
                speedReadText.Stop();
                speedReadAll.Stop();
                RDNIDLib.disconnectCardRD(obj);
                RDNIDLib.deselectReaderRD(obj);
                return res;
            }

            speedReadText.Stop();

            String NIDData = aByteToString(data);
            if (NIDData == "")
            {
                lblStatus.Text = "Status: Read Text error";
                MessageBox.Show("Read Text error");
            }
            else
            {
                string[] fields = NIDData.Split('#');

                m_txtID.Text = fields[(int)NID_FIELD.NID_Number];                             // or use m_txtID.Text = fields[(int)NID_FIELD.NID_Number];

                String fullname = fields[(int)NID_FIELD.TITLE_T] + " " +
                                    fields[(int)NID_FIELD.NAME_T] + " " +
                                    fields[(int)NID_FIELD.MIDNAME_T] + " " +
                                    fields[(int)NID_FIELD.SURNAME_T].TrimEnd();  //Remove space
                m_txtFullNameT.Text = fullname;

                fullname = fields[(int)NID_FIELD.TITLE_E] + " " +
                                    fields[(int)NID_FIELD.NAME_E] + " " +
                                    fields[(int)NID_FIELD.MIDNAME_E] + " " +
                                    fields[(int)NID_FIELD.SURNAME_E].TrimEnd();  //Remove space
                ;
                m_txtFullNameE.Text = fullname;

                m_txtBrithDate.Text = _yyyymmdd_(fields[(int)NID_FIELD.BIRTH_DATE]);


                string txtAddrees = "";

                if (fields[(int)NID_FIELD.HOME_NO] != "") txtAddrees += fields[(int)NID_FIELD.HOME_NO] + "   ";
                if (fields[(int)NID_FIELD.MOO] != "") txtAddrees += fields[(int)NID_FIELD.MOO] + "   ";
                if (fields[(int)NID_FIELD.TROK] != "") txtAddrees += fields[(int)NID_FIELD.TROK] + "   ";
                if (fields[(int)NID_FIELD.SOI] != "") txtAddrees += fields[(int)NID_FIELD.SOI] + "   ";
                if (fields[(int)NID_FIELD.ROAD] != "") txtAddrees += fields[(int)NID_FIELD.ROAD] + "   ";
                if (fields[(int)NID_FIELD.TUMBON] != "") txtAddrees += fields[(int)NID_FIELD.TUMBON] + "   ";
                if (fields[(int)NID_FIELD.AMPHOE] != "") txtAddrees += fields[(int)NID_FIELD.AMPHOE] + "   ";
                if (fields[(int)NID_FIELD.PROVINCE] != "") txtAddrees += fields[(int)NID_FIELD.PROVINCE].TrimEnd();

                m_txtAddress.Text = txtAddrees;

                if (fields[(int)NID_FIELD.GENDER] == "1")
                {
                    m_txtGender.Text = "ชาย";
                }
                else if (fields[(int)NID_FIELD.GENDER] == "2")
                {
                    m_txtGender.Text = "หญิง";
                }

                m_txtIssueDate.Text = _yyyymmdd_(fields[(int)NID_FIELD.ISSUE_DATE]);
                m_txtExpiryDate.Text = _yyyymmdd_(fields[(int)NID_FIELD.EXPIRY_DATE]);
                if ("99999999" == m_txtExpiryDate.Text)
                    m_txtExpiryDate.Text = "99999999 ตลอดชีพ";
                m_txtIssueNum.Text = fields[(int)NID_FIELD.ISSUE_NUM];
                m_txtIssueplace.Text = fields[(int)NID_FIELD.ISSUE_PLACE].TrimEnd();  //Remove space                
            }

            byte[] NIDPicture = new byte[1024 * 5];
            int imgsize = NIDPicture.Length;
            res = RDNIDLib.getNIDPhotoRD(obj, NIDPicture, out imgsize);
            if (res != DefineConstants.NID_SUCCESS)
                if (res != DefineConstants.NID_SUCCESS)
                {
                    String m;
                    m = String.Format("Error: {0} ", res);

                    lblStatus.Text = "Status: Error " + res;
                    MessageBox.Show(m, Application.ProductName);
                    speedReadAll.Stop();
                    RDNIDLib.disconnectCardRD(obj);
                    RDNIDLib.deselectReaderRD(obj);
                    return res;
                }
            speedReadAll.Stop();



            lbl_Time.Text = string.Format("Read Text: {0} s, Text + Photo: {1} s", speedReadText.Elapsed.TotalSeconds.ToString("f2"), speedReadAll.Elapsed.TotalSeconds.ToString("f2"));



            byte[] byteImage = NIDPicture;
            if (byteImage == null)
            {
                lblStatus.Text = "Status: Read Photo error";
                MessageBox.Show("Read Photo error", Application.ProductName);
            }
            else
            {
                //m_picPhoto
                Image img = Image.FromStream(new MemoryStream(byteImage));
                Bitmap MyImage = new Bitmap(img, m_picPhoto.Width - 2, m_picPhoto.Height - 2);
                m_picPhoto.Image = (Image)MyImage;
                lblStatus.Text = "Status: Ready";
            }

            RDNIDLib.disconnectCardRD(obj);
            RDNIDLib.deselectReaderRD(obj);
            return 0;
        }

        private void OnReadCardClick(object sender, EventArgs e)
        {
            lbl_Time.Text = "";
            m_txtID.Text = "";
            m_txtFullNameT.Text = "";
            m_txtFullNameE.Text = "";
            m_txtAddress.Text = "";
            m_txtBrithDate.Text = "";
            m_txtGender.Text = "";
            m_txtIssueDate.Text = "";
            m_txtExpiryDate.Text = "";
            m_txtIssueNum.Text = "";
            m_txtIssueplace.Text = "";
            m_picPhoto.Image = null;
            lblStatus.Text = "Status: Reading...";


            ReadCard();
        }

        private void btnloadLIC_Click(object sender, EventArgs e)
        {
            int UploadReturn = 0;
            lblStatus.Text = "Status: Downloading...";
            this.Refresh();
            lbl_Time.Text = "";
            byte[] _lic = String2Byte(StartupPath + "\\RDNIDLib.DLX");
            int res = RDNIDLib.updateLicenseFileRD(_lic);
            UploadReturn = res;
            if (res < 0)
            {
                string s = string.Format("Error: {0}", res);
                lblStatus.Text = "Status: Error " + res;
                MessageBox.Show(s, Application.ProductName);
            }
            else
            {
                //ReOpenlib After updateLicenseFile For Update License Info.

                //If the Library is opened Please close Library first.
                //closelib.
                res = RDNIDLib.closeNIDLibRD();

                //ReOpenlib.           
                int nres = RDNIDLib.openNIDLibRD(_lic);                     //for openLib 
                if (nres != 0)
                {
                    String m;
                    m = String.Format("Error: {0} ", nres);
                    MessageBox.Show(m);
                }

                byte[] Licinfo = new byte[1024];
                RDNIDLib.getLicenseInfoRD(Licinfo);
                m_lblDLDInfo.Text = "License Info: " + aByteToString(Licinfo) + " (" + UploadReturn + ")";
                lblStatus.Text = "Status: Ready";                           
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            m_txtID.Text = "";
            m_txtFullNameT.Text = "";
            m_txtFullNameE.Text = "";
            m_txtAddress.Text = "";
            m_txtBrithDate.Text = "";
            m_txtGender.Text = "";
            m_txtIssueDate.Text = "";
            m_txtExpiryDate.Text = "";
            m_txtIssueNum.Text = "";
            m_picPhoto.Image = null;
        }

        private void m_ListReaderCard_Click(object sender, EventArgs e)
        {
            lbl_Time.Text = "";
            m_ListReaderCard.ResetText();
            m_ListReaderCard.Items.Clear();
            ListCardReader();
        }

        private void m_ListReaderCard_TextChanged(object sender, EventArgs e)
        {
            lbl_Time.Text = "";
            m_txtID.Text = "";
            m_txtFullNameT.Text = "";
            m_txtFullNameE.Text = "";
            m_txtAddress.Text = "";
            m_txtBrithDate.Text = "";
            m_txtGender.Text = "";
            m_txtIssueDate.Text = "";
            m_txtExpiryDate.Text = "";
            m_txtIssueNum.Text = "";           
            m_txtIssueplace.Text = "";
            m_picPhoto.Image = null;
        }


    }
}
